package com.web.api.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSONObject;
import com.web.api.model.dto.AnswerDto;
import com.web.api.model.po.AnswerExcel;
import com.web.api.model.po.JobTypeEntity;
import com.web.api.model.po.UserInfo;
import com.web.api.service.AnswerService;
import com.web.api.service.impl.AnswerServiceImpl;
import com.web.api.service.mail.MailServiceImpl;
import com.web.api.utils.Constant;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.CookieStore;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author 王玉鹏
 * @version 1.0
 * @className ImportExcelListener
 * @description TODO
 * @date 2021/9/24 17:57
 */
@Slf4j
@Component
public class ImportExcelListener extends AnalysisEventListener<AnswerExcel> {

    private AnswerService answerService;

    private MailServiceImpl mailService;

    List<AnswerExcel> list = new ArrayList<>();

    String safeCode = "1111";
    int errorCount = 0;
    StringBuffer msg = new StringBuffer();

    public ImportExcelListener(AnswerService answerService, MailServiceImpl mailService) {
        super();
        this.answerService = answerService;
        this.mailService = mailService;
    }

    @Override
    public void invoke(AnswerExcel answerExcel, AnalysisContext analysisContext) {
        log.info("解析到一条数据:{}", JSONObject.toJSON(answerExcel));
        try {
            doAnswer(answerExcel);
        } catch (Exception e) {
            log.error("发生异常", e);
        }
        list.add(answerExcel);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        if (ObjectUtils.isEmpty(msg)) {
            msg.append("账户全部导入成功,已开始批量答题");
        }
        String html = "<html><body>" +"<h1>共计导入"+list.size()+"条数据，成功"+(list.size()-errorCount)+"条，失败"+errorCount+"条，具体详情如下：</h1><p>"+ msg.toString() + "</p></html></body>";
        try {
            mailService.sendSimpleMail(new String[]{"251664727@qq.com", "632062365@qq.com"}, list.size() + "条数据全部导入完毕，开始批量答题", html);
        } catch (Exception e) {
            log.error("发送邮件异常", e);
        }
        log.info("{}条数据全部导入完毕，开始批量答题！", list.size());
    }

    public void doAnswer(AnswerExcel answerExcel) {
        //登录答题系统
        CookieStore cookieStore = null;
        try {
            cookieStore = answerService.login(answerExcel.getUserName(), answerExcel.getPassword(), safeCode);
        } catch (Exception e) {
            errorCount++;
            msg.append("失败用户：" + answerExcel.getName() + " 工号: " +answerExcel.getUserName()+ "，失败原因：" + e.getMessage() + "</br>");
            return;
        }
        //根据工种名称获取工种基本信息拿到工种id和关卡初始id
        JobTypeEntity jobTypeEntity = Constant.jobTypeMap.get(answerExcel.getJobTypeName());
        if (ObjectUtils.isEmpty(jobTypeEntity)) {
            errorCount++;
            msg.append("失败用户：" + answerExcel.getName() + " 工号: " +answerExcel.getUserName()+ "，失败原因：工种填写错误</br>");
            return;
        }
        AnswerDto answerDto = new AnswerDto();
        answerDto.setJobNo(answerExcel.getUserName());
        answerDto.setJobTypeName(answerExcel.getJobTypeName());
        answerDto.setTargetScore(answerExcel.getTargetScores());
        answerDto.setJobTypeId(jobTypeEntity.getTypeId());
        answerDto.setJobGkId(jobTypeEntity.getGkId());
        answerDto.setBatchFlag(true);
        answerDto.setLevel(3);
        //开始答题
        try {
            answerService.answer(cookieStore, answerDto);
        } catch (Exception e) {
            errorCount++;
            msg.append("失败用户：" + answerExcel.getUserName() + "，失败原因：" + e.getMessage() + "</br>");
            return;
        }
    }
}
