package com.web.api.service.impl;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.web.api.controller.answer.croe.HttpRequestConfig;
import com.web.api.exception.ErrorCode;
import com.web.api.exception.SgException;
import com.web.api.model.dto.AnswerDto;
import com.web.api.model.po.UserInfo;
import com.web.api.model.po.SgEntity;
import com.web.api.utils.Constant;
import com.web.api.utils.ContextUtils;
import com.web.api.utils.HttpClientUtils;
import com.web.api.utils.SgUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.CookieStore;
import org.apache.http.client.protocol.HttpClientContext;
import org.apache.http.protocol.HttpContext;
import org.slf4j.MDC;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.*;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static com.web.api.utils.Constant.MOBILE_API_URL;
import static com.web.api.utils.Constant.SERVER_IP;

/**
 * @author 王玉鹏
 * @version 1.0
 * @className AnswerManager
 * @description 答题核心逻辑类
 * @date 2020/3/19 13:14
 */
@Slf4j
@Component
@EnableAsync
public class AnswerManager {
    /**
     * 金币，砖石，积分
     */
    public final static Pattern USER_PATTENT = Pattern.compile("(<div class=\"aui-gird-lable aui-font-size-15\" style=\"font-weight:bold\">)(.*?)(</div>)");
    /**
     * 用户名
     */
    public final static Pattern NAME_PATTENT = Pattern.compile("(<div class=\"aui-list-item-text text-white aui-font-size-18\">)(.*?)(</div>)");
    /**
     * 用户头像
     */
    public final static Pattern IMAGE_PATTENT = Pattern.compile("(<img src=\")(.*?)(\" class=\"aui-img-round\" >)");
    /**
     * 当前关卡
     */
    public final static Pattern PASS_PATTENT = Pattern.compile("(<div class=\"aui-badge\" style=\"color: #83755e;)(.*?)\"(>)(.*?)(</div>)");


    @Async("asyncServiceExecutor")
    public void anwser(CookieStore cookie, SgEntity sgEntity, AnswerDto answerDto, Double startScore) throws Exception {
        log.info("当前活动线程数量:" + Thread.activeCount());
        MDC.put("userJobNo", answerDto.getJobNo());
        HttpContext httpContext = new HttpClientContext();
        ((HttpClientContext) httpContext).setCookieStore(cookie);
        int gs = sgEntity.getCurrentGs();
        //外层循环控制1到10关，内层循环控制对应关数多少题数
        UserInfo userInfo = ContextUtils.getUserInfo(answerDto.getJobNo());
        for (int i = 1; i <= 10; i++) {
            if (!ContextUtils.getOnOff(answerDto.getJobNo())) {
                log.info("当前用户:{},已发起停止答题请求", answerDto.getJobNo());
                break;
            }
            sgEntity.setCurrentGs(gs);
            userInfo.setCurrentGk(gs);
            //根据当前工种id,难度,以及关卡进度,计算当前闯关页面id
            int num = gs + answerDto.getJobGkId();
            sgEntity.setFinalId(num);
            //打开当前闯关页面（开始答题计时）
            openAnswerHtml(httpContext, sgEntity, userInfo);
            //开始进行当前关卡答题
            for (int j = 1; j <= Constant.answerNum.get(gs) + 10; j++) {
                if (!ContextUtils.getOnOff(answerDto.getJobNo())) {
                    log.info("当前用户:{},已发起停止答题请求", answerDto.getJobNo());
                    break;
                }
                if (!ContextUtils.getStatus()) {
                    log.info("前端发起一键终止请求，用户：{}答题结束，结束所有答题即将结束，：", answerDto.getJobNo());
                    throw new SgException(ErrorCode.STOP_All_ANSWER.getCode(), ErrorCode.STOP_All_ANSWER.getName());
                }
                log.info("{}====================>第{}题", answerDto.getJobTypeName(), j);
                //循环答题（每2秒一次 ）
                JSONObject res = startAnswer(httpContext, sgEntity, 3);
                Thread.sleep(2000);

                if (Integer.valueOf(res.get("code").toString()) == -1) {
                    log.error("答题速度过快，直接进行下一题答题");
                    continue;
                }
                if (Integer.valueOf(res.get("right_num").toString()) >= Constant.answerNum.get(gs)) {
                    log.error("当前关已答题{}道", Integer.valueOf(res.get("right_num").toString()));
                    break;
                }
            }
            if (ContextUtils.getOnOff(answerDto.getJobNo())) {
                //对当前关卡进行结算
                log.info("开始第" + sgEntity.getCurrentGs() + "关结算，" + "当前答题页面id：" + num + "闯关页面id:{}", sgEntity.getFinalId());
                String answerResult;
                String settlementUrl = "cglb/passresult?passid=" + sgEntity.getFinalId() + "&matchid=0";
                try {
                    answerResult = SgUtils.sgRequest("post", httpContext, settlementUrl);
                } catch (IOException e) {
                    log.error("结算异常,{},e");
                    Thread.sleep(21000);
                    log.info("再次发起结算请求……");
                    answerResult = SgUtils.sgRequest("post", httpContext, settlementUrl);
                    log.info("结算成功");
                }
                if (StringUtils.isEmpty(answerResult)) {
                    log.error("结算返回数据为空，再次发起结算请求");
                    answerResult = SgUtils.sgRequest("post", httpContext, settlementUrl);
                }
                if (answerResult.contains("只差一点点")) {
                    log.error("错误题目过多，闯关失败……");
                }
                //结算完成刷新个人信息答题成绩
                refreshUserRecord(httpContext, answerDto, startScore, userInfo);
                gs++;
                if (gs > 10) {
                    log.info("一轮答题结束");
                    break;
                }
            }
        }
        if (!ContextUtils.getOnOff(answerDto.getJobNo())) {
            UserInfo latestUserinfo = getLatestUserInfo(httpContext);
            userInfo.setFinalScore(latestUserinfo.getIntegral());
            throw new SgException(ErrorCode.STOP_ANSWER.getCode(), ErrorCode.STOP_ANSWER.getName());
        }
        //一轮答题10关结束后，开始从第一关开始
        sgEntity.setCurrentGs(1);
        anwser(cookie, sgEntity, answerDto, startScore);
    }

    /**
     * 打开闯关页面
     */
    private static void openAnswerHtml(HttpContext httpContext, SgEntity sgEntity, UserInfo userInfo) throws Exception {

        //根据难度选择计算关卡id变化
        switch (sgEntity.getDifficultyLevel()) {
            case 1:
                break;
            case 2:
                sgEntity.setFinalId(sgEntity.getFinalId() + 10);
                break;
            case 3:
                sgEntity.setFinalId(sgEntity.getFinalId() + 20);
                break;
            default:
                break;
        }
        log.info("开始第{}关答题,答题页面id：{}", sgEntity.getCurrentGs(), sgEntity.getFinalId());

        String openResult = SgUtils.sgRequest("post", httpContext, "cglb/dt?id=" + sgEntity.getFinalId() + "&matchid=0");

        if (StringUtils.isEmpty(openResult)) {
            log.error("请求返回结果集为空:{}，请求地址为：{}", MOBILE_API_URL + "cglb/dt?id=" + sgEntity.getFinalId() + "&matchid=0");
            UserInfo latestUserinfo = getLatestUserInfo(httpContext);
            userInfo.setFinalScore(latestUserinfo.getIntegral());
            throw new SgException(ErrorCode.RESULT_EMPTY.getCode(), ErrorCode.RESULT_EMPTY.getName());
        }
        if (openResult.contains("今天的闯关时间已用尽")) {
            log.error("今天的闯关时间已用尽");
            UserInfo latestUserinfo = getLatestUserInfo(httpContext);
            userInfo.setFinalScore(latestUserinfo.getIntegral());
            throw new SgException(ErrorCode.TIME_Limit.getCode(), ErrorCode.TIME_Limit.getName());
        }
        if (openResult.contains("未解锁")) {
            log.error("当前关卡未解锁");
            UserInfo latestUserinfo = getLatestUserInfo(httpContext);
            userInfo.setFinalScore(latestUserinfo.getIntegral());
            throw new SgException(ErrorCode.LEVEL_NOT_UNLOCKED.getCode(), ErrorCode.LEVEL_NOT_UNLOCKED.getName());
        }
    }

    /**
     * 获取当前用户关卡进度
     */
    public static int getCurrentLevel(CookieStore cookie, int level, int jobType) throws Exception {
        String scjl = SgUtils.sgRequest("post", cookie, "cglb/gk/?id=" + jobType + "&passtype=" + level + "&matchid=0");
        Matcher m = PASS_PATTENT.matcher(scjl);
        String currentGs = "";
        while (m.find()) {
            currentGs = m.group(4);
        }
        if (currentGs == "") {
            throw new SgException(ErrorCode.GK_EMPTY.getCode(), ErrorCode.GK_EMPTY.getName());
        }
        log.info("当前答题记录关数为:【{}】", currentGs);
        return Integer.parseInt(currentGs);
    }

    /**
     * 获取用户基本信息
     */
    public static UserInfo makeUserInfo(String html) throws Exception {
        UserInfo userInfoDto = new UserInfo();
        Matcher m = USER_PATTENT.matcher(html);
        String str;
        int a = 0;
        while (m.find()) {
            str = m.group(2);
            if (a == 0) {
                //金币
                userInfoDto.setGoldCoin(str);
            }
            if (a == 1) {
                //钻石
                userInfoDto.setDiamond(str);
            }
            if (a == 2) {
                //积分
                userInfoDto.setIntegral(Double.parseDouble(str));
            }
            a++;
        }
        //获取用户名
        Matcher m2 = NAME_PATTENT.matcher(html);
        while (m2.find()) {
            log.info("当前登陆用户：{}", m2.group(2));
            userInfoDto.setUserName(m2.group(2));
        }
        //获取用户头像
        Matcher m3 = IMAGE_PATTENT.matcher(html);
        String src;
        while (m3.find()) {
            src = m3.group(2);
            log.info("当前用户头像：{}", SERVER_IP + src);
            userInfoDto.setImage(SERVER_IP + src);
        }
        return userInfoDto;
    }

    /**
     * 刷新个人记录
     */
    private void refreshUserRecord(HttpContext httpContext, AnswerDto answerDto, Double startScore, UserInfo userInfo) throws Exception {
        UserInfo latestUserinfo = getLatestUserInfo(httpContext);
        userInfo.setFinalScore(latestUserinfo.getIntegral());
        userInfo.setIntegral(latestUserinfo.getIntegral());
        if (latestUserinfo.getIntegral() - startScore > answerDto.getTargetScore()) {
            log.info("已完成{}分刷分,如需继续请重新开始", answerDto.getTargetScore());
            throw new SgException(ErrorCode.SORCE_LIMT.getCode(), ErrorCode.SORCE_LIMT.getName());
        }
    }

    /**
     * 单次答题
     */
    private JSONObject startAnswer(HttpContext httpContext, SgEntity sgEntity, int retry) throws Exception {
        int ram = new Random().nextInt(50);
        String stem = Constant.stemList.get(ram);
        log.info("当前随机选择数字为：{}，题干为:【{}】", ram, stem);
        HttpRequestConfig answerConfig = HttpRequestConfig.create().url(MOBILE_API_URL + "cglb/ajaxdt?passid=" + sgEntity.getFinalId() + "&" + stem).context(httpContext);
        String answerResult = HttpClientUtils.post(answerConfig).getResponseText();
        JSONObject jb = null;
        try {
            jb = JSONObject.parseObject(answerResult);
        } catch (JSONException exception) {
            if (exception.getMessage().contains("重新登录")) {
                log.error("账号已在别处登陆");
                throw new SgException(ErrorCode.LOGOUT.getCode(), ErrorCode.LOGOUT.getName());
            } else if (exception.getMessage().contains("页面错误") && retry >= 0) {
                log.info("retry:{}", retry);
                retry--;
                log.error("页面错误,重新发起请求……,cookie:{}", ContextUtils.getUserStringCookie());
                Thread.sleep(10000);
                jb = startAnswer(httpContext, sgEntity, retry);
                if (retry == 0) {
                    throw new SgException(ErrorCode.USER_INFO_ERROR.getCode(), ErrorCode.USER_INFO_ERROR.getName());
                }
            } else {
                log.error("json解析异常{}", exception.getMessage());
            }
        }
        return jb;

    }

    private static UserInfo getLatestUserInfo(HttpContext httpContext) throws Exception {
        String data = SgUtils.sgRequest("post", httpContext, "users");
        if (data.contains("页面错误")) {
            log.error("获取个人记录出错");
            throw new SgException(ErrorCode.USER_INFO_ERROR.getCode(), ErrorCode.USER_INFO_ERROR.getName());
        }
        UserInfo latestUserInfo = AnswerManager.makeUserInfo(data);
        return latestUserInfo;
    }

//    public static void main(String[] args) {
//        HttpRequest httpRequest =new HttpRequest(MOBILE_API_URL + "cglb/ajaxdt?passid=231&id=142803&answer=ABC");
//        httpRequest.cookie("BJ83T0U0lFrKq1=20848;safetycode1=2940");
//        HttpResponse httpResponse = httpRequest.method(Method.GET).execute();
//        System.out.println(httpResponse.body());
//    }
}
